package com.ruoyi.web.work.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.web.work.api.util.AppUtil;
import com.ruoyi.web.work.domain.Pay;
import com.ruoyi.web.work.domain.User;
import com.ruoyi.web.work.domain.dto.PayAuditDto;
import com.ruoyi.web.work.domain.dto.PayDto;
import com.ruoyi.web.work.mapper.PayMapper;
import com.ruoyi.web.work.service.IPayService;
import com.ruoyi.web.work.service.IUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.rmi.ServerException;
import java.util.List;

/**
 * @author lsw
 * @date 2024-07-01
 */
@Service
public class PayServiceImpl extends ServiceImpl<PayMapper, Pay> implements IPayService {
    @Autowired
    private PayMapper payMapper;

    @Autowired
    IUserService userService;

    @Override
    public List<Pay> selectList(Pay pay) {
        return payMapper.selectList(pay);
    }

    @Override
    public Pay selectByNums(String nums) {
        return payMapper.selectByNums(nums);
    }

    @Override
    public AjaxResult cashOut(PayDto dto) throws ServerException {
        User user = userService.getById(AppUtil.getUser().getId());
        if (dto.getMoney().compareTo(user.getMoney()) > 0) {
            return AjaxResult.error("提现金额超过账户余额");
        }
        if (StringUtils.isEmpty(user.getBankName()) || StringUtils.isEmpty(user.getBankAccount())) {
            return AjaxResult.error("银行卡信息未完善,无法提现");
        }
        //保存提现记录
        Pay pay = new Pay();
        pay.setUserId(AppUtil.getUser().getId());
        pay.setMoney(dto.getMoney());
        pay.setState(0);
        pay.setType(2);
        pay.setTitle("余额提现");
        pay.setVersion(System.currentTimeMillis());
        pay.setNums(StringUtils.generateNumber());
        if (!save(pay)) {
            throw new ServerException("提现申请失败,请联系平台");
        }
        //更新账户余额
        user.setMoney(user.getMoney().subtract(pay.getMoney()));
        if (!userService.updateById(user)) {
            throw new ServerException("更新账户余额失败,请联系平台");
        }
        return AjaxResult.success();
    }

    @Transactional
    @Override
    public AjaxResult audit(PayAuditDto dto) throws ServerException {
        Pay pay = getById(dto.getId());
        if (pay == null) {
            return AjaxResult.error("提现信息不存在");
        }
        if (pay.getState() == 1) {
            return AjaxResult.error("不能重复操作");
        }
        pay.setState(dto.getState());
        pay.setMsg(dto.getMsg());
        if (!updateById(pay)) {
            throw new ServerException("提现失败,请联系平台");
        }
        //提现驳回需要退回到用户账户余额
        if (dto.getState() == 2) {
            User user =userService.getById(pay.getUserId());
            user.setMoney(user.getMoney().add(pay.getMoney()));
            if (!userService.updateById(user)) {
                throw new ServerException("更新用户账户余额失败,请联系平台");
            }
        }
        return AjaxResult.success();
    }
}
